#include <conio.h>
#include <stdlib.h>

#include <pr.h>
#include <prgui.h>

#ifdef __3DFX__
#include <pr3dfx.h>
#include <glide.h>
#endif

#ifdef WIN32
#include <winutil.h>
#endif

#include <prdev.h>

/* Power Render Example 11

   Description:

   Shows how to display a rectangular overlay texture using any material.
   This is useful for cockpits, text, and sprites.

   The method is to allocate a sprite object, which has 4 vertices.
   We can then set up the transformed vertex state ourselves and place the
   screen coordinates anywhere.  Clipped overlays are supported in this
   version.  Since this method uses texture triangle, it works on both
   software and 3D hardware.  Also, any rendering methods can be used.

   This example defaults to 640x480.

   Sorting Issues:

   You may want to show more than one overlay, such as a cockpit with
   individual controls over it.  To make sure zbuffering works correctly,
   you can choose the z value constant of the overlay.  With software
   rendering, sorting only has whole value precision, so values must
   be at least 1.0 apart.

   Software Issues:

   There are faster ways of showing overlays in software, which do not use
   texture mapping.  If you plan on using rectangular overlays which could
   be simply scaled or non-scaled bitmaps, use WGT's wputblock or wresize
   commands.

*/

                                             
PR_VIEWPORT viewport;                  /* Our viewport structure */
PR_CAMERA *camera;                     /* One camera */
PR_DWORD vwidth, vheight;


#define OVERLAY_MATERIAL 0
PR_OBJECT *shape;
PR_ENTITY *overlay;

#define NUMSPR 15
typedef struct {
 PR_DWORD x, y;            /* Center */
 PR_DWORD xscale, yscale; 
 PR_DWORD xdir, ydir, xscaledir, yscaledir;
} sprite_info;
sprite_info sprites[NUMSPR];



void InitializeOverlayMaterial (void)
{
PR_MATERIAL *m;

  PR_SetTextureFormat (TEXTURE_XRAY);
    m = &PR_ObjectMaterialList[OVERLAY_MATERIAL];
  PR_SetMaterialName (m, "overlay sprite");
  PR_SetMaterialMethod (m, T_XTEXTURED);
  PR_SetMaterialBaseColor (m, 0);
  PR_SetMaterialColor (m, 255, 255, 255);
  PR_SetMaterialTexture (m, PR_LoadTexture("car.pcx"));
  PR_SetMaterialTable (m, 0);
  PR_SetMaterialShades (m, 31);
  PR_SetMaterialEnvironmentMap (m, 0);
  PR_SetMaterialMipMapState (m, FALSE);
  PR_SetMaterialAlpha (m, 128);

  shape = PR_AllocSprite (OVERLAY_MATERIAL);
  overlay = PR_CreateEntity (shape, "overlay");
}



/*
This is a private function in Power Render.  It takes a pointer to
a vertex data structure and a pointer to the far clipping plane, and
generates clipping codes for the vertex.  This is needed to ensure
polygons off the screen will be drawn correctly.  Clipping on the
near plane is not computed, so make sure the z value is >= 1.0.
*/

#ifdef __WATCOMC__
#pragma aux clipcheck "_*" parm caller [esi] [edi] modify exact [eax ebx ecx edx esi edi];
#endif
extern void clipcheck (PR_VERTEX_DATA *vert, PR_REAL *farclip);



/* This routine could be modifed to allow any 4 screen coordinates, so you
   could have rotated/warped images.
   For this demonstration it assumes the image is rectangular.  */

void ShowOverlay (PR_ENTITY *overlay,
                  PR_DWORD x0, PR_DWORD y0, PR_DWORD x1, PR_DWORD y1, PR_REAL z)
{
PR_VERTEX_DATA *vdata;
PR_REAL *farclip;

PR_REAL z_times_ooxscale;
PR_REAL z_times_ooyscale;

  z_times_ooxscale = z * (1.0f / active_viewport.xscale);
  z_times_ooyscale = z * (1.0f / active_viewport.yscale);

  /* Make sure you have a camera set up otherwise the far clip plane code
     may be generated by mistake */
  farclip = &PR_CurrentCamera->farclip;

  overlay->shape->segment_list[0].flags |= FLAG_VISIBLE;

  /* First vertex is the center and we can ignore it, since it
     is only used for sprite culling

     The basic idea here is that in order to clip, we need to
     undo the perspective transform on the screen coordinates given as
     parameters.  The vx, vy, and vz elements of the vertex data structure
     are initialized so the clipcheck routine can generate clipping
     codes from the homogenous clipping volume.  

   */

  vdata = overlay->shape->segment_list[0].vertex_list[1].vdata;
  vdata->sx = x0;
  vdata->sy = y1;
  vdata->vx = (vdata->sx - active_viewport.centerx) * z_times_ooxscale;
  vdata->vy = (active_viewport.centery - vdata->sy) * z_times_ooyscale;
  vdata->vz = z;
  clipcheck (vdata, farclip);

  vdata = overlay->shape->segment_list[0].vertex_list[2].vdata;
  vdata->sx = x1;
  vdata->sy = y1;
  vdata->vx = (vdata->sx - active_viewport.centerx) * z_times_ooxscale;
  vdata->vy = (active_viewport.centery - vdata->sy) * z_times_ooyscale;
  vdata->vz = z;
  clipcheck (vdata, farclip);

  vdata = overlay->shape->segment_list[0].vertex_list[3].vdata;
  vdata->sx = x1;
  vdata->sy = y0;
  vdata->vx = (vdata->sx - active_viewport.centerx) * z_times_ooxscale;
  vdata->vy = (active_viewport.centery - vdata->sy) * z_times_ooyscale;
  vdata->vz = z;
  clipcheck (vdata, farclip);

  vdata = overlay->shape->segment_list[0].vertex_list[4].vdata;
  vdata->sx = x0;
  vdata->sy = y0;
  vdata->vx = (vdata->sx - active_viewport.centerx) * z_times_ooxscale;
  vdata->vy = (active_viewport.centery - vdata->sy) * z_times_ooyscale;
  vdata->vz = z;
  clipcheck (vdata, farclip);

  PR_RenderEntity (overlay);
}



void InitializeSprites (void)
/* Set up a number of sprites with random values */
{
PR_DWORD i;

  for (i = 0; i < NUMSPR; i++)
    {
     sprites[i].x = rand () % vwidth;
     sprites[i].y = rand () % vheight;
     sprites[i].xscale = (rand () % 60) + 10;
     sprites[i].yscale = (rand () % 60) + 10;

     sprites[i].xdir = (rand () % 2) ? -1 : 1;
     sprites[i].ydir = (rand () % 2) ? -1 : 1;
     sprites[i].xscaledir = (rand () % 2) ? -1 : 1;
     sprites[i].yscaledir = (rand () % 2) ? -1 : 1;
    }
}


void MoveSprites (void)
/* Make them bounce around */
{
PR_DWORD i;

  for (i = 0; i < NUMSPR; i++)
    {
     sprites[i].x += sprites[i].xdir * 5;
     sprites[i].y += sprites[i].ydir * 5;
     sprites[i].xscale += sprites[i].xscaledir * 2;
     sprites[i].yscale += sprites[i].yscaledir * 2;

     if (sprites[i].x < 0)
       sprites[i].xdir *= -1;
     if (sprites[i].y < 0)
       sprites[i].ydir *= -1;
     if (sprites[i].xscale < 10)
       sprites[i].xscaledir *= -1;
     if (sprites[i].yscale < 10)
       sprites[i].yscaledir *= -1;

     if (sprites[i].x > vwidth)
       sprites[i].xdir *= -1;
     if (sprites[i].y > vheight)
       sprites[i].ydir *= -1;
     if (sprites[i].xscale > 70)
       sprites[i].xscaledir *= -1;
     if (sprites[i].yscale > 70)
       sprites[i].yscaledir *= -1;

       ShowOverlay (overlay,
                sprites[i].x - sprites[i].xscale,
                sprites[i].y - sprites[i].yscale,
                sprites[i].x + sprites[i].xscale,
                sprites[i].y + sprites[i].yscale,
                1 + (PR_REAL)i / 100);
    }
}




void main (int argc, char *argv[])
{
PR_UDWORD BLACK;                       /* Color index/value for black */

  ChooseDevice ();
  
  if (device == DEVICE_VGA)
    device = DEVICE_SVGA;

  PR_Initialize (1000);                /* Maximum 1000 triangles per frame */

  InitializeDevices ();

  PR_AllocMaterials (32);              /* Allocate the materials, */
  PR_AllocTextures (32);               /* textures, */
  PR_AllocShadeTables (2);             /* and shade tables */

  /* Try 640x480 first */
  vwidth = 640;
  vheight = 480;

  if (PR_SetMode (vwidth, vheight, 60) != MODE_SUCCESS)
    {
     /* If that doesn't work, try the first one in the mode list */
     vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
     vheight = PR_VideoModes[0].height;
     PR_SetMode (vwidth, vheight, 60);
    }

  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */

  camera = PR_AllocCamera ();          /* Initialize the camera */
  PR_InitializeCamera (camera);
  PR_SetActiveCamera (camera);

  InitializeOverlayMaterial ();
  InitializeSprites ();

#if !defined (MSGLIDE) && !defined (WTGLIDE) && !defined (MSGL) && !defined (WTGL)
  /* Set the palette from the textures loaded */
  wsetpalette (0, 255, global_palette);
#endif

  PR_OpenScreen (PR_BACKBUFFER);
  BLACK = PRGFX_MakeColor (0, 0, 0);

  installkbd ();
#ifndef WIN32
  minit ();
#endif

  PR_OpenScreen (PR_BACKBUFFER);

  /* main program loop */
  while (!kbdon[KEY_ESC])
    {
     #ifdef WIN32
       UpdateMessages ();
     #endif      

     #if defined (MSDD) || defined (WTDD) || defined (MSGL) || defined (WTGL)
       if (PR_LostSurface)
         PR_RestoreAll ();
     #endif

     PR_OpenScreen (PR_BACKBUFFER);
     PR_NewFrame ();                   /* Begin a new frame */

     PRGFX_SetColor (BLACK);
     PRGFX_ClearScreen ();             /* Clear the last frame */

     MoveSprites ();
     ShowOverlay (overlay, -150 + mouse.mx, -140 + mouse.my, 150 + mouse.mx, 140 + mouse.my, 1.5);

     PR_RenderFrame ();                /* Draw the scene */

     PR_Flip (1);                      /* Show the background page */
    }

  uninstallkbd ();
  mdeinit ();

  /* Free all allocated memory */
  PR_CloseViewport (&viewport);
  PR_DeleteAllShadeTables ();
  PR_DeleteAllTextures ();
  PR_DeleteAllMaterials ();
  PR_ShutDown ();

#ifndef WIN32
  wsetmode (3);                        /* Go back to text mode */
#endif
}

